/* ***********************************************************************
 * File name:
 * 		rawnetif.h
 * 
 * Description:
 * 		This file contains all the wrapper for interfacing with
 *      the RawNetIo driver
 * 	
 * Revision:
 * ---------------------------------------------------------------------
 * 		2003-11-22	xpankajx@intellectualheaven.com
 *		Intial version
 *
 *		2005-04-24	xpankajx@intellectualheaven.com
 *		Converting this code to a DLL
 * ---------------------------------------------------------------------
 * 	Copyright (c) : Pankaj Garg
 * **********************************************************************/
 
 /**
 * @addtogroup	RawNetIf Raw Network IO Interface Dll
 *
 * @brief		Provides functions for interfacing with RawNetIo Driver
 *
 * 				This DLL provides the functions which are required to interface
 * 				with the RawNetIo driver. A user mode application can use this
 * 				DLL to bind the RawNetIo driver to a particular physical 
 * 				ethernet adapter and then send/receive raw ethernet frames. 
 * 				This DLL can be used to easily implement various network 
 * 				applications such as network sniffer, network intrusion 
 * 				detection etc
 * 			
 * 
 * Copyright (c) : Pankaj Garg & www.intellectualheaven.com
 *
 * @note		Please include the file rawnetif.h to use these functions
 * 
 * @warning	    The RawNetIo driver only works on Windows 2000, XP and 2003.
 * 				So this DLL should be used on those operating systems only.
 * 
 * @author	    Pankaj Garg
 *
 * @{
 **/

#ifndef _RAWNETIF_H_
#define _RAWNETIF_H_

// System-Includes!!!
#include <setupapi.h>
#include <cfgmgr32.h>
#include <devguid.h>
#include <regstr.h>
#include <string>
#include <vector>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief	Stores the information for an adapter
 *
 *			This structure is used to store information about an ethernet
 *			adapter. This information is retrieved by calling function
 *			RnIf_AdapterEnumerate and can be used to bind to and open an
 *			adapter for doing raw IO on that adapter
 **/
struct RNIF_ADAPTER_INFO
{
	//! Adapter name in "/Device/{GUID}" format
	WCHAR		DeviceName[MAX_PATH];

	//! Adapter symbolic link in "/DosDevices/Global/{GUID}" format
	WCHAR		SymbolicLink[MAX_PATH];

	//! A user friendly adapter name that can be used for display purpose
	WCHAR		FriendlyName[MAX_PATH];

};

//! Pointer type to an RNIF_ADAPTER_INFO struct
typedef RNIF_ADAPTER_INFO *PRNIF_ADAPTER_INFO;

//! A list for RNIF_ADAPTER_INFO struct
struct RNIF_ADAPTER_INFO_LIST
{
	//! Array of RNIF_ADAPTER_INFO structs
	PRNIF_ADAPTER_INFO		AdapterArray;

	//! Current number of valid elements in the array
	USHORT					CurrentCount;

	//! Maximum number of elements that can be accomodated in the array
	USHORT					MaximumCount;
};


/**
 * @brief	Installs and starts the RawNetIo driver
 *
 *			This function installs the RawNetIo driver in the system. and starts
 * 			it if the installation is successful.
 * 
 * @warning	Please make sure that you supply a unique instance name because if
 *			another	driver is using the same name then this function would fail
 * 			and the RawNetIo driver will not be installed.
 *
 * @param	UniqueInstanceName - User should supply a unique name for the driver
 * 			driver. Usually very few driver instances are active at a time so
 * 			it is recommended that you select a unique name for your application
 * 			at compile time and always use that	name in you application. It is
 * 			also recommented that you don't allow users to run two instances of
 * 			your application otherwise a name conflict would occur. If it is a
 * 			requirement to allow two instance of same application both using
 * 			RawNetIo then you should generate a	unique string for the instance
 * 			name at run-time. Please don't use only numbers or a guid for
 * 			instance name.
 * 	
 * @return	If the function succeed, the return value is = TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_DriverInitInstance(
	wchar_t *UniqueInstanceName);

/**
 * @brief	Stops the RawNetIo driver and removes it from the system
 *
 *			This function stops the RawNetIo driver and removes it from the
 * 			system. Once you remove the driver, you need to call 
 *			RnIf_DriverInitInstance to install and use the driver again.
 *
 * @return	If the function succeed, the return value is = TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_DriverExitInstance();


/**
 * @brief	Used to check if the driver is running or not
 *
 *			This function can be used to check if the driver is installed and
 *			running or not. If a driver is not running then you should call
 *			RnIf_DriverInitInstance before accessing driver functionality.
 *
 * @return	If the driver is loaded, the return value is = TRUE.
 *			If the driver is not loaded, the return value is FALSE.
 **/
BOOL
__stdcall
RnIf_DriverIsRunning();

/**
 * @brief	Enumerate the list of adapters
 *
 *			This function enumerates the list of active adapters in the system.
 *			The calling	application can use this information in  function
 *			RnIf_AdapterBind to bind to a particular adapter and function
 *			RnIf_AdapterOpen to open that adapter for Raw IO.
 *
 * @param	AdapterList - This is a list of RNIF_ADAPTER_INFO struct that
 * 			stores the information about all the adapters enumerated.
 *			Caller should allocate memory for the AdapterArray member of
 *			AdapterList and caller should also initialize MaximumCount to
 *			maximum number of elements in AdapterArray and CurrentCount to 0.
 * 	
 * @return	If the function succeed, the return value is = TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_AdapterEnumerate(
	RNIF_ADAPTER_INFO_LIST& AdapterList);


/**
 * @brief	Bind to a given adapter
 *
 *			This function tells the driver to bind to a particular adapter. The
 *			driver should be bound to a particular adapter before you try to
 *			open that adapter using function RnIf_AdapterOpen.
 *
 * @param	inAdapterInfo - Information about the adapter
 * 	
 * @return	If the function succeed, the return value is TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_AdapterBind(
	RNIF_ADAPTER_INFO& inAdapterInfo);


/**
 * @brief	Open a given adapter
 *
 *			This function opens an adapter for reading/writing ethernet frames
 *			on it and carry out other operations. An application should pass
 *			one of the adapters retrieved by RnIf_AdapterEnumerate to this
 *			function. Also note that this function may fail, if you didn't bind
 *			the adapter to RawNetIo driver by calling RnIf_AdapterBind.
 *
 * @param	inAdapterInfo - Information about the adapter
 * 	
 * @return	If the function succeed, the return value is a handle to the 
 *			adapter.
 *			If the function fails, the return value is INVALID_HANDLE_VALUE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
HANDLE
__stdcall
RnIf_AdapterOpen(
	RNIF_ADAPTER_INFO& inAdapterInfo);


/**
 * @brief	Close an open adapter
 *
 *			This function closes an adapter which was previously opened using
 *			RnIf_AdapterOpen.
 *
 * @note	If an application changed the mode of the adapterto promiscuous,
 *			then it should reset the mode to directed before closing the handle
 *			to the adapter. Leaving the adapter mode to	promiscuous will cause
 *			severe degradation of performance.
 *
 * @param	hAdapter - HANDLE to the adapter
 * 	
 * @return	None
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
void
__stdcall
RnIf_AdapterClose(
	HANDLE hAdapter);


/**
 * @brief	Puts the adapter in directed mode
 *
 *			This function puts an adapter in directed mode. In directed mode an
 *			adapter can only send/receive packets which have the corresponding
 *			source/destination MAC address of its own.
 *
 * @param	hAdapter - HANDLE to the adapter
 * @param	Overlapped - OVERLAPPED structure with valid hEvent. Set all other
 *				struct members to zero.
 * 	
 * @return	If the function succeed, the return value is TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_AdapterSetModeDirected(
	HANDLE			hAdapter,
	LPOVERLAPPED	Overlapped);


/**
 * @brief	Puts the adapter in promiscuous mode
 *
 *			This function puts an adapter in promiscuous mode. In this mode an
 *			adapter can send/receive packets with any source/destination MAC
 *			address. In this mode the ethernet adapter hands over all the packet
 *			received by it even if it doesn't belong to it. This mode can be
 *			used to sniff all the network traffic that is passing by the 
 *			adapter.
 *
 * @warning	Promiscuous mode causes the adapter to receive significantly more
 *			network traffic than directed mode and thus it should be used
 *			cautiously. Also before closing an adapter, make sure that you
 *			change the adapter mode back to directed, otherwise you would
 *			un-necessarily keep the adapter receiving more packets.
 *
 * @param	hAdapter - HANDLE to the adapter
 * @param	Overlapped - OVERLAPPED structure with valid hEvent. Set all other
 *				struct members to zero.
 * 	
 * @return	If the function succeed, the return value is TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_AdapterSetModePromiscuous(
	HANDLE			hAdapter,
	LPOVERLAPPED	Overlapped);


/**
 * @brief	Sends a raw buffer to the adapter
 *
 *			This function can be used to send raw buffer or raw ethernet frames
 *			to the underlying physical adapter (or ethernet card). If you
 *			intend to send packets with source MAC address different from the
 *			MAC address of underlying adapter then put that adapter in
 *			promiscuous mode before calling this function. You don't need to
 *			put the adapter in promiscuous mode, if you only want to send
 *			packets with adapter source MAC address.
 *
 * @param	hAdapter - HANDLE to the adapter
 * @param	Buffer - address of the buffer which points to the data to send
 * @param	Length - length of the buffer to send
 * @param	BytesSent - The number of bytes actually sent
 * @param	Overlapped - OVERLAPPED struct with valid hEvent value. Set all
 *				other struct members to zero.
 * @param	Sync - Boolean variable to indicate if the call should be executed
 *				synchrnously or asynchrnously. If Sync is FALSE, then the
 *				function returns immediately and caller should call 
 *				GetOverlappedResult to get the result of Send operation.
 * 	
 * @return	If the function succeed, the return value is TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_PacketSend(
	HANDLE			hAdapter,
	LPVOID			Buffer,
	ULONG			Length,
	PULONG			BytesSent,
	LPOVERLAPPED	Overlapped,
	BOOL			Sync);


/**
 * @brief	Receives an incoming packet from the adapter
 *
 *			This function is used to receive an incoming packet from the
 *			underlying adapter synchronously or asynchronously. Set the Sync
 *			parameter to TRUE to execute the function synchrnously.
 *
 * @warning	If the buffer supplied is less than the packet size, then the
 *			remaining bytes in the packet are dropped on the floor. A caller
 *			should usually send at least 1514 bytes buffer to make sure that
 *			the packet is fully received.
 *
 * @param	hAdapter - HANDLE to the adapter
 * @param	Buffer - address of the buffer which receives incoming packet
 * @param	Length - length of the buffer
 * @param	BytesReceived - The number of bytes actually received
 * @param	Overlapped - OVERLAPPED struct with valid hEvent value. Set all
 *				other struct members to zero.
 * @param	Sync - Boolean variable to indicate if the call should be executed
 *				synchrnously or asynchrnously. If Sync is FALSE, then the
 *				function returns immediately and caller should call 
 *				GetOverlappedResult to see how many bytes are received.
 *
 * @warning	If you use this function to send the packet asyncronously, then make
 *			sure that the Overlapped parameter remains valid when you call
 *			GetOverlappedResult. If this parameter goes out of scope then it
 *			will result in unpredictable behavior.
 * 	
 * @return	If the function succeed, the return value is TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_PacketReceive(
	HANDLE			hAdapter,
	LPVOID			Buffer,
	ULONG			Length,
	PULONG			BytesReceived,
	LPOVERLAPPED	Overlapped,
	BOOL			Sync);


/**
 * @brief	Receives an incoming packet from the adapter asynchronously only
 *
 *			This function can be used to receive an incoming packet from the
 *			underlying adapter asynchrnously. Unlike RnIf_PacketReceive, in this
 *			function, the caller doesn't have to check the status of the
 *			operation by calling GetOverlappedResult, instead the system calls
 *			the CompletionRoutine callback handler once a new packet is
 *			received. An application should typically queue 256-512 read
 *			requests in a loop and as the packets are received, it can process
 *			them in the CompletionRoutine. This is the recommended mode if you
 *			wish to receive packets asynchronously. Also this mode is
 *			recommended over synchronous mode for better throughput.
 *			
 *
 * @param	hAdapter - HANDLE to the adapter
 * @param	Buffer - address of the buffer which receives incoming packet
 * @param	Length - length of the buffer
 * @param	Overlapped - OVERLAPPED struct. hEvent member is not used in this
 *				API so caller can use it to store their own context. Offset and
 *				OffsetHigh are also not used so an application can use them as
 *				well to store its context information. Set all the other members
 *				to zero.
 * @param	CompletionRoutine - A function which would be called when the
 *				packet is received. The function would be called in the context
 *				of the thread, which calls RnIf_PacketReceiveEx.
 *
 * @warning	If you use this function to send the packet asyncronously, then make
 *			sure that the Overlapped parameter remains valid when you call
 *			GetOverlappedResult. If this parameter goes out of scope then it
 *			will result in unpredictable behavior.
 *
 * @note	This function internally calls ReadFileEx so please refer to 
 *			<B>ReadFileEx</B> in MSDN for detailed information. Also refer to
 *			<B>FileIOCompletionRoutine</B> in MSDN for information about the
 *			CompletionRoutine function.
 * 	
 * @return	If the function succeed, the return value is TRUE.
 *			If the function fails, the return value is FALSE.
 *
 * @remarks	In case of failure, call GetLastError for more information.
 **/
BOOL
__stdcall
RnIf_PacketReceiveEx(
	HANDLE							hAdapter,
	LPVOID							Buffer,
	ULONG							Length,
	LPOVERLAPPED					Overlapped,
	LPOVERLAPPED_COMPLETION_ROUTINE	CompletionRoutine);

#ifdef __cplusplus
}
#endif

#endif

/** @} **/
